from numpy import linspace, sum

def simpson(f, a, b, N):
    r"""
    Simpson quadrature of function f from a to b with N subintervals.

    f:     Function f(x)
    a, b:  Bounds of the integration interval
    N:     Number of subintervals
    """
    x, h = linspace(a, b, 2*N+1, retstep=True)
    # quadrature weights:
    #   boundary nodes: w=1/3
    #   internal nodes: w=2/3
    #   midpoint nodes: w=4/3
    I = h/3.0 * sum(f(x[:-2:2]) + 4.0*f(x[1:-1:2]) + f(x[2::2]))
    return I


if __name__ == "__main__":
    from numpy import arange, size, zeros, log, polyfit
    from scipy import integrate

    # Define a function and an interval:
    f = lambda x: 1.0 / (1.0 + (5*x)**2)
    left = 0.0
    right = 1.0

    # Exact integration with scipy.integrate.quad:
    exact, e = integrate.quad(f, left, right)

    # Simpson rule for different number of quadrature points
    N = arange(2, 101)
    res = zeros(size(N))
    for i, n in enumerate(N):
        res[i] = simpson(f, left, right, n)

    err = abs(res - exact)

    # Linear fit to determine convergence order
    p = polyfit(log(N[-50:]), log(err[-50:]), 1)
    print(("Convergence order: %f" % -p[0]))
